@@ -112,10 +112,12 @@ testmap = [
     RegrTest('test_asynchat.py', usemodules='select fcntl'),
     RegrTest('test_asyncio'),
     RegrTest('test_asyncore.py', usemodules='select fcntl'),
+    RegrTest('test_asyncgen.py'),
     RegrTest('test_atexit.py', core=True),
     RegrTest('test_audioop.py'),
     RegrTest('test_augassign.py', core=True),
     RegrTest('test_base64.py', usemodules='struct'),
+    RegrTest('test_baseexception.py'),
     RegrTest('test_bigaddrspace.py'),
     RegrTest('test_bigmem.py'),
     RegrTest('test_binascii.py', usemodules='binascii'),
@@ -187,6 +189,7 @@ testmap = [
     RegrTest('test_descrtut.py', core=True),
     RegrTest('test_devpoll.py'),
     RegrTest('test_dict.py', core=True),
+    RegrTest('test_dict_version.py', skip="implementation detail"),
     RegrTest('test_dictcomps.py', core=True),
     RegrTest('test_dictviews.py', core=True),
     RegrTest('test_difflib.py'),
@@ -195,6 +198,7 @@ testmap = [
     RegrTest('test_doctest.py', usemodules="thread"),
     RegrTest('test_doctest2.py'),
     RegrTest('test_docxmlrpc.py'),
+    RegrTest('test_dtrace.py'),
     RegrTest('test_dummy_thread.py', core=True),
     RegrTest('test_dummy_threading.py', core=True),
     RegrTest('test_dynamic.py'),
@@ -207,6 +211,7 @@ testmap = [
     RegrTest('test_eof.py', core=True),
     RegrTest('test_epoll.py'),
     RegrTest('test_errno.py', usemodules="errno"),
+    RegrTest('test_exception_hierarchy.py'),
     RegrTest('test_exception_variations.py'),
     RegrTest('test_exceptions.py', core=True),
     RegrTest('test_extcall.py', core=True),
@@ -242,6 +247,7 @@ testmap = [
     RegrTest('test_getopt.py', core=True),
     RegrTest('test_getpass.py'),
     RegrTest('test_gettext.py'),
+    RegrTest('test_generator_stop.py'),
     RegrTest('test_glob.py', core=True),
     RegrTest('test_global.py', core=True),
     RegrTest('test_grammar.py', core=True),
@@ -378,6 +384,7 @@ testmap = [
     RegrTest('test_sched.py'),
     RegrTest('test_scope.py', core=True),
     RegrTest('test_script_helper.py'),
+    RegrTest('test_secrets.py'),
     RegrTest('test_select.py'),
     RegrTest('test_selectors.py'),
     RegrTest('test_set.py', core=True),
@@ -404,18 +411,21 @@ testmap = [
     RegrTest('test_statistics.py'),
     RegrTest('test_strftime.py'),
     RegrTest('test_string.py', core=True),
+    RegrTest('test_string_literals.py'),
     RegrTest('test_stringprep.py'),
-    RegrTest('test_strlit.py', core=True),
+    RegrTest('test_strlit.py'),
     RegrTest('test_strptime.py'),
     RegrTest('test_strtod.py'),
     RegrTest('test_struct.py', usemodules='struct'),
     RegrTest('test_structmembers.py', skip="CPython specific"),
     RegrTest('test_structseq.py'),
+    RegrTest('test_subclassinit.py'),
     RegrTest('test_subprocess.py', usemodules='signal'),
     RegrTest('test_sunau.py'),
     RegrTest('test_sundry.py'),
     RegrTest('test_super.py', core=True),
     RegrTest('test_support.py'),
+    RegrTest('test_symbol.py'),
     RegrTest('test_symtable.py', skip="implementation detail"),
     RegrTest('test_syntax.py', core=True),
     RegrTest('test_sys.py', core=True, usemodules='struct'),
@@ -456,9 +466,12 @@ testmap = [
     RegrTest('test_unary.py', core=True),
     RegrTest('test_unicode.py', core=True),
     RegrTest('test_unicode_file.py'),
+    RegrTest('test_unicode_file_functions.py'),
+    RegrTest('test_unicode_identifiers.py'),
     RegrTest('test_unicodedata.py'),
     RegrTest('test_unittest.py', core=True),
     RegrTest('test_univnewlines.py'),
+    RegrTest('test_utf8source.py'),
     RegrTest('test_unpack.py', core=True),
     RegrTest('test_unpack_ex.py', core=True),
     RegrTest('test_urllib.py'),
@@ -481,6 +494,7 @@ testmap = [
     RegrTest('test_weakref.py', core=True, usemodules='_weakref'),
     RegrTest('test_weakset.py'),
     RegrTest('test_webbrowser.py'),
+    RegrTest('test_winconsoleio.py'),
     RegrTest('test_winreg.py'),
     RegrTest('test_winsound.py'),
     RegrTest('test_with.py'),
@@ -491,6 +505,7 @@ testmap = [
     RegrTest('test_xml_etree_c.py'),
     RegrTest('test_xmlrpc.py'),
     RegrTest('test_xmlrpc_net.py'),
+    RegrTest('test_yield_from.py'),
     RegrTest('test_zipapp.py'),
     RegrTest('test_zipfile.py'),
     RegrTest('test_zipfile64.py'),
